@@ -31,7 +31,10 @@ score = function(x, data, type = NULL, ..., debug = FALSE) {
 AIC.bn = function(object, data, ..., k = 1) {
 
   # parameter sanitization done in the score() function.
-  score(object, data = data, type = "aic", k = k, ...)
+  if (is.data.discrete(data))
+    score(object, data = data, type = "aic", k = k, ...)
+  else
+    score(object, data = data, type = "aic-g", k = k, ...)
 
 }#AIC.bn
 
@@ -40,78 +43,13 @@ AIC.bn = function(object, data, ..., k = 1) {
 logLik.bn = function(object, data, ...) {
 
   # parameter sanitization done in the score() function.
-  score(x = object, data = data, type = "loglik", ...)
+  if (is.data.discrete(data))
+    score(x = object, data = data, type = "loglik", ...)
+  else
+    score(x = object, data = data, type = "loglik-g", ...)
 
 }#LOGLIK.BN
 
-# measure the strength of the arcs in a directed graph.
-arc.strength = function(x, data, criterion = NULL, ..., debug = FALSE) {
-
-  # check x's class.
-  check.bn(x)
-  # arc strength is undefined in partially directed graphs.
-  if (is.pdag(x$arcs, names(x$nodes)))
-    stop("the graph is only partially directed.")
-  # check the data are there.
-  check.data(data)
-  # check the network against the data
-  check.bn.vs.data(x, data)
-  # check debug.
-  check.logical(debug)
-  # check criterion.
-  if (is.null(criterion)) {
-
-    # if no criterion is specified use either the default one or the
-    # one used by the learning algorithm.
-    if (x$learning$test == "none")
-      criterion = check.test(criterion, data)
-    else
-      criterion = x$learning$test
-
-  }#THEN
-  else {
-
-    criterion = check.criterion(criterion, data)
-
-  }#ELSE
-
-  # expand and sanitize score-specific arguments and the alpha threshold.
-  if (criterion %in% available.tests) {
-
-    # sanitize the alpha threshold.
-    alpha = check.alpha(list(...)$alpha, network = x)
-
-    # sanitize B (the number of bootstrap/permutation samples).
-    B = check.B(list(...)$B, criterion)
-
-    # warn about unused arguments.
-    check.unused.args(list(...), c("alpha", "B"))
-
-    res = arc.strength.test(network = x, data = data, alpha = alpha,
-      test = criterion, B = B, debug = debug)
-
-    # add extra information for strength.plot().
-    res = structure(res, mode = "test", threshold = alpha)
-
-  }#THEN
-  else {
-
-    # expand and sanitize score-specific arguments.
-    extra.args = check.score.args(score = criterion, network = x,
-                 data = data, extra.args = list(...))
-
-    res = arc.strength.score(network = x, data = data, score = criterion,
-      extra = extra.args, debug = debug)
-
-    # add extra information for strength.plot().
-    res = structure(res, mode = "score", threshold = 0)
-
-  }#ELSE
-
-  return(structure(res, class = c("bn-strength", class(res))))
-
-}#ARC.STRENGTH
-
 # infer the direction of an ipothetic arc between two specified nodes.
 choose.direction = function(x, arc, data, criterion = NULL, ..., debug = FALSE) {
 
@@ -134,7 +72,7 @@ choose.direction = function(x, arc, data, criterion = NULL, ..., debug = FALSE)
       criterion = x$learning$test
 
   }#THEN
-  else {
+  else if (!identical(criterion, "bootstrap")) {
 
     criterion = check.criterion(criterion, data)
 
@@ -168,6 +106,16 @@ choose.direction = function(x, arc, data, criterion = NULL, ..., debug = FALSE)
           extra.args = extra.args, debug = debug)
 
   }#ELSE
+  else if (criterion == "bootstrap") {
+
+    # expand and check bootstrap-specific arguments.
+    extra.args = check.bootstrap.args(list(...), network = x, data = data)
+
+    x = choose.direction.boot(x, data = data, arc = arc, 
+          extra.args = extra.args, algorithm = extra.args[["algorithm"]], 
+          algorithm.args = extra.args[["algorithm.args"]], debug = debug)
+
+  }#THEN
 
   invisible(x)
 
