@@ -83,7 +83,10 @@ smaller = function(a, b) {
 # present in the 'data' argument.
 configurations = function(data) {
 
-  factor(.Call("cfg", data = data, PACKAGE = "bnlearn"))
+  x = .Call("cfg", data = data, PACKAGE = "bnlearn")
+
+  factor(x = x, levels = unique.default(x), ordered = FALSE)
+
 
 }#CONFIGURATIONS
 
